@@ -1,6 +1,8 @@
 context("ms_dissolve")
-library(geojsonio)
-# library(sp)
+suppressPackageStartupMessages({
+  library("geojsonio")
+  library("sp")
+})
 
 poly <- structure('{"type":"FeatureCollection",
   "features":[
@@ -119,3 +121,28 @@ test_that("copy_fields and sum_fields works", {
 "geo_json"))
   expect_equal(ms_dissolve(poly_attr, sum_fields = c("a", "b")), expected_out)
 })
+
+## sf classes
+if (suppressPackageStartupMessages(require("sf", quietly = TRUE))) {
+  points_sf <- st_as_sf(points_spdf)
+  poly_sf <- st_as_sf(poly_spdf)
+
+  test_that("ms_dissolve.sf works with points", {
+    expect_is(ms_dissolve(points_sf), "sf")
+    expect_is(ms_dissolve(st_geometry(points_sf)), "sfc")
+  })
+
+  test_that("ms_dissolve.sf works with polygons", {
+    expect_is(ms_dissolve(poly_sf), "sf")
+    expect_is(ms_dissolve(st_geometry(poly_sf)), "sfc")
+  })
+
+  test_that("weight argument works", {
+    expect_error(ms_dissolve(points, weight = "w"), "APIError")
+    expect_error(ms_dissolve(points_sf, weight = "w"), "specified 'weight' column not present in input data")
+    expect_gt(sum(sf::st_coordinates(ms_dissolve(points_sf, weight = "foo"))),
+              sum(sf::st_coordinates(ms_dissolve(points_sf))))
+    expect_gt(sum(sp::coordinates(ms_dissolve(points_spdf, weight = "foo"))),
+              sum(sp::coordinates(ms_dissolve(points_spdf))))
+  })
+}
