@@ -1,5 +1,5 @@
 context("ms_filter_fields")
-library(geojsonio)
+suppressPackageStartupMessages(library("geojsonio"))
 
 poly <- structure("{\"type\":\"FeatureCollection\",
 \"features\":[{\"type\":\"Feature\",
@@ -58,3 +58,13 @@ test_that("ms_filter_fields fails correctly", {
   expect_error(ms_filter_fields(geojson_sp(poly), "d"), "Not all named fields exist in input data")
   expect_error(ms_filter_fields(poly, 1), "fields must be a character vector")
 })
+
+if (suppressPackageStartupMessages(require("sf", quietly = TRUE))) {
+  test_that("ms_filter_fields works with sf", {
+    lines_sf <- read_sf(lines)
+    out_sf <- ms_filter_fields(lines_sf, c("a", "b"))
+    expect_is(out_sf, "sf")
+    expect_equal(names(out_sf), c("a", "b", "geometry"))
+    expect_error(ms_filter_fields(lines_sf, "d", "Not all fields are in input"))
+  })
+}
\ No newline at end of file
